@@ -14,6 +14,7 @@ from collective.easyform.api import is_file_data
 from collective.easyform.api import lnbr
 from collective.easyform.api import OrderedDict
 from collective.easyform.config import FORM_ERROR_MARKER
+from collective.easyform.config import HAS_XLSX_SUPPORT
 from collective.easyform.interfaces import IAction
 from collective.easyform.interfaces import IActionFactory
 from collective.easyform.interfaces import ICustomScript
@@ -24,8 +25,8 @@ from copy import deepcopy
 from csv import writer as csvwriter
 from datetime import date
 from datetime import datetime
-from datetime import timedelta
 from DateTime import DateTime
+from datetime import timedelta
 from decimal import Decimal
 from email import encoders
 from email.header import Header
@@ -40,6 +41,7 @@ from logging import getLogger
 from plone import api
 from plone.app.textfield.value import RichTextValue
 from plone.autoform.view import WidgetsView
+from plone.registry.interfaces import IRegistry
 from plone.supermodel.exportimport import BaseHandler
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone.utils import safe_unicode
@@ -47,16 +49,17 @@ from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
 from Products.PythonScripts.PythonScript import PythonScript
 from six import BytesIO
 from six import StringIO
+from tempfile import NamedTemporaryFile
 from time import time
 from xml.etree import ElementTree as ET
 from z3c.form.interfaces import DISPLAY_MODE
+from zope.component import getUtility
 from zope.component import queryUtility
 from zope.contenttype import guess_content_type
 from zope.interface import implementer
 from zope.schema import Bool
 from zope.schema import getFieldsInOrder
 from zope.security.interfaces import IPermission
-
 import six
 
 
@@ -76,7 +79,7 @@ class ActionFactory(object):
         self.kw = kw
 
     def available(self, context):
-        """ field is addable in the current context """
+        """field is addable in the current context"""
         securityManager = getSecurityManager()
         permission = queryUtility(IPermission, name=self.permission)
         if permission is None:
@@ -84,7 +87,7 @@ class ActionFactory(object):
         return bool(securityManager.checkPermission(permission.title, context))
 
     def editable(self, field):
-        """ test whether a given instance of a field is editable """
+        """test whether a given instance of a field is editable"""
         return True
 
     def __call__(self, *args, **kw):
@@ -95,8 +98,31 @@ class ActionFactory(object):
 
 @implementer(IAction)
 class Action(Bool):
-    """Base action class.
-    """
+    """Base action class."""
+
+    def serialize(self, field):
+        """Serializa field to save in various formats, like XML, CSV, etc."""
+        if field is None:
+            return ""
+        if isinstance(field, (set, list, tuple)):
+            list_value = list([self.serialize(f) for f in field])
+            return dumps(list_value)
+        if isinstance(field, dict):
+            dict_value = {str(key): self.serialize(val) for key, val in field.items()}
+            return dumps(dict_value)
+        if isinstance(field, RichTextValue):
+            return field.raw
+        if isinstance(field, datetime):
+            return field.strftime("%Y/%m/%d, %H:%M:%S")
+        if isinstance(field, date):
+            return field.strftime("%Y/%m/%d")
+        if isinstance(field, timedelta):
+            return str(field)
+        if isinstance(field, (int, float, Decimal, bool)):
+            return str(field)
+        if isinstance(field, six.string_types):
+            return safe_unicode(field)
+        return safe_unicode(repr(field))
 
     def onSuccess(self, fields, request):
         raise NotImplementedError(
@@ -105,8 +131,7 @@ class Action(Bool):
 
 
 class DummyFormView(WidgetsView):
-    """A dummy form to get the widgets rendered for the mailer action.
-    """
+    """A dummy form to get the widgets rendered for the mailer action."""
 
     mode = DISPLAY_MODE
     ignoreContext = True
@@ -123,8 +148,7 @@ class Mailer(Action):
         super(Mailer, self).__init__(**kw)
 
     def get_portal_email_address(self, context):
-        """Return the email address defined in the Plone site.
-        """
+        """Return the email address defined in the Plone site."""
         return api.portal.get_registry_record("plone.email_from_address")
 
     def secure_header_line(self, line):
@@ -139,8 +163,7 @@ class Mailer(Action):
         return line
 
     def get_mail_body(self, unsorted_data, request, context):
-        """Returns the mail-body with footer.
-        """
+        """Returns the mail-body with footer."""
         schema = get_schema(context)
 
         form = DummyFormView(context, request)
@@ -185,8 +208,7 @@ class Mailer(Action):
         return template.pt_render(extra_context=extra)
 
     def get_owner_info(self, context):
-        """Return owner info.
-        """
+        """Return owner info."""
         pms = getToolByName(context, "portal_membership")
         ownerinfo = context.getOwner()
         ownerid = fullname = ownerinfo.getId()
@@ -207,8 +229,7 @@ class Mailer(Action):
         return (fullname, toemail)
 
     def get_addresses(self, fields, request, context, from_addr=None, to_addr=None):
-        """Return addresses.
-        """
+        """Return addresses."""
         # get Reply-To
         reply_addr = None
         if hasattr(self, "replyto_field"):
@@ -249,8 +270,7 @@ class Mailer(Action):
         return (to, from_addr, reply_addr)
 
     def get_subject(self, fields, request, context):
-        """Return subject.
-        """
+        """Return subject."""
         # get subject header
         nosubject = u"(no subject)"  # TODO: translate
         subject = None
@@ -333,40 +353,15 @@ class Mailer(Action):
 
         return headerinfo
 
-    def serialize(self, field):
-        """Serializa field to save to XML.
-        """
-        if field is None:
-            return ""
-        if isinstance(field, (set, list, tuple)):
-            list_value = list([self.serialize(f) for f in field])
-            return dumps(list_value)
-        if isinstance(field, dict):
-            dict_value = {str(key): self.serialize(val) for key, val in field.items()}
-            return dumps(dict_value)
-        if isinstance(field, RichTextValue):
-            return field.raw
-        if isinstance(field, datetime):
-            return field.strftime("%Y/%m/%d, %H:%M:%S")
-        if isinstance(field, date):
-            return field.strftime("%Y/%m/%d")
-        if isinstance(field, timedelta):
-            return str(field)
-        if isinstance(field, (int, float, Decimal, bool)):
-            return str(field)
-        if isinstance(field, six.string_types):
-            return safe_unicode(field)
-        return safe_unicode(repr(field))
-
     def get_attachments(self, fields, request):
-        """Return all attachments uploaded in form.
-        """
+        """Return all attachments uploaded in form."""
 
         attachments = []
 
         # if requested, generate CSV attachment of form values
         sendCSV = getattr(self, "sendCSV", None)
-        if sendCSV:
+        sendXLSX = getattr(self, "sendXLSX", None)
+        if sendCSV or sendXLSX:
             csvdata = ()
         sendXML = getattr(self, "sendXML", None)
         if sendXML:
@@ -375,7 +370,7 @@ class Mailer(Action):
         for fname in fields:
             field = fields[fname]
 
-            if sendCSV:
+            if sendCSV or sendXLSX:
                 if not is_file_data(field) and (
                     getattr(self, "showAll", True) or fname in showFields
                 ):
@@ -412,6 +407,28 @@ class Mailer(Action):
             # Set MIME type of attachment to 'application' so that it will be encoded with base64
             attachments.append((filename, "application/csv", "utf-8", csv))
 
+        if sendXLSX:
+            from openpyxl import Workbook
+
+            wb = Workbook()
+            ws = wb.active
+            ws.append(csvdata)
+
+            with NamedTemporaryFile() as tmp:
+                wb.save(tmp.name)
+                output = tmp.read()
+
+            now = DateTime().ISO().replace(" ", "-").replace(":", "")
+            filename = "formdata_{0}.xlsx".format(now)
+            attachments.append(
+                (
+                    filename,
+                    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
+                    "utf-8",
+                    output
+                )
+            )
+
         if sendXML:
             # use ET.write to get a proper XML Header line
             output = BytesIO()
@@ -426,8 +443,7 @@ class Mailer(Action):
         return attachments
 
     def get_mail_text(self, fields, request, context):
-        """Get header and body of e-mail as text (string)
-        """
+        """Get header and body of e-mail as text (string)"""
         headerinfo = self.get_header_info(fields, request, context)
         body = self.get_mail_body(fields, request, context)
         if six.PY2 and isinstance(body, six.text_type):
@@ -496,8 +512,7 @@ class Mailer(Action):
         return outer.as_string()
 
     def onSuccess(self, fields, request):
-        """e-mails data.
-        """
+        """e-mails data."""
         context = get_context(self)
         mailtext = self.get_mail_text(fields, request, context)
         host = api.portal.get_tool(name="MailHost")
@@ -627,36 +642,69 @@ class SaveData(Action):
         """
         return list(self._storage.items())
 
+    def get_header_row(self):
+        titles = self.getColumnTitles()
+        encoded_titles = []
+        for t in titles:
+            if six.PY2 and isinstance(t, six.text_type):
+                t = t.encode("utf-8")
+            encoded_titles.append(t)
+        return encoded_titles
+
+    def get_row_data(self, row):
+        names = self.getColumnNames()
+
+        def get_data(row, i):
+            data = row.get(i, "")
+            if is_file_data(data):
+                data = data.filename
+            if six.PY2 and isinstance(data, six.text_type):
+                return data.encode("utf-8")
+            return data
+
+        return [get_data(row, i) for i in names]
+
     def getSavedFormInputForEdit(self, header=False, delimiter=","):
         """Returns saved as CSV text"""
         sbuf = StringIO()
+
+        if len(delimiter) == 0:
+            registry = getUtility(IRegistry)
+            delimiter = registry.get("easyform.csv_delimiter").encode("utf-8")
+
         writer = csvwriter(sbuf, delimiter=delimiter)
-        names = self.getColumnNames()
-        titles = self.getColumnTitles()
 
         if header:
-            encoded_titles = []
-            for t in titles:
-                if six.PY2 and isinstance(t, six.text_type):
-                    t = t.encode("utf-8")
-                encoded_titles.append(t)
-            writer.writerow(encoded_titles)
+            writer.writerow(self.get_header_row())
+
         for row in self.getSavedFormInput():
+            writer.writerow(self.get_row_data(row))
 
-            def get_data(row, i):
-                data = row.get(i, "")
-                if is_file_data(data):
-                    data = data.filename
-                if six.PY2 and isinstance(data, six.text_type):
-                    return data.encode("utf-8")
-                return data
-
-            row_data = [get_data(row, i) for i in names]
-            writer.writerow(row_data)
         res = sbuf.getvalue()
         sbuf.close()
         return res
 
+    def get_saved_form_input_as_xlsx(self, header=False):
+        assert (
+            HAS_XLSX_SUPPORT
+        ), "XLSX export not suppored, please enable 'downloadxlsx' extra"
+
+        from openpyxl import Workbook
+
+        wb = Workbook()
+        ws = wb.active
+
+        if header:
+            ws.append(self.get_header_row())
+
+        for row in self.getSavedFormInput():
+            ws.append(list(map(self.serialize, self.get_row_data(row))))
+
+        output = BytesIO()
+        wb.save(output)
+        result = output.getvalue()
+        return result
+
     def getColumnNames(self):
         # """Returns a list of column names"""
         context = get_context(self)
@@ -690,7 +738,7 @@ class SaveData(Action):
                 names.append(IExtraData[f].title)
         return names
 
-    def download_csv(self, response):
+    def download_csv(self, response, delimiter):
         # """Download the saved data as csv
         # """
         response.setHeader(
@@ -699,7 +747,7 @@ class SaveData(Action):
         )
         response.setHeader("Content-Type", "text/comma-separated-values")
         value = self.getSavedFormInputForEdit(
-            getattr(self, "UseColumnNames", False), delimiter=","
+            getattr(self, "UseColumnNames", False), delimiter=delimiter
         )
         if isinstance(value, six.text_type):
             value = value.encode("utf-8")
@@ -720,15 +768,35 @@ class SaveData(Action):
             value = value.encode("utf-8")
         response.write(value)
 
-    def download(self, response):
+    def download_xlsx(self, response):
+        # """Download the saved data as tsv
+        # """
+        response.setHeader(
+            "Content-Disposition",
+            'attachment; filename="{0}.xlsx"'.format(self.__name__),
+        )
+
+        response.setHeader(
+            "Content-Type",
+            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
+        )
+        value = self.get_saved_form_input_as_xlsx(
+            getattr(self, "UseColumnNames", False)
+        )
+
+        response.write(value)
+
+    def download(self, response, delimiter=""):
         # """Download the saved data
         # """
         format = getattr(self, "DownloadFormat", "tsv")
         if format == "tsv":
             return self.download_tsv(response)
+        elif format == "xlsx":
+            return self.download_xlsx(response)
         else:
             assert format == "csv", "Unknown download format"
-            return self.download_csv(response)
+            return self.download_csv(response, delimiter)
 
     def itemsSaved(self):
         return len(self._storage)
@@ -757,8 +825,7 @@ class SaveData(Action):
         storage[id] = value
 
     def onSuccess(self, fields, request):
-        """Saves data.
-        """
+        """Saves data."""
         # if LP_SAVE_TO_CANONICAL and not loopstop:
         # LinguaPlone functionality:
         # check to see if we're in a translated
