@@ -5,6 +5,7 @@ from collective.easyform.interfaces import IEasyForm
 from collective.easyform.interfaces import IEasyFormForm
 from collective.easyform.interfaces import IFieldExtender
 from collective.easyform.interfaces import ILabel
+from collective.easyform.interfaces import IHCaptcha
 from collective.easyform.interfaces import INorobotCaptcha
 from collective.easyform.interfaces import IReCaptcha
 from collective.easyform.interfaces import IRichLabel
@@ -29,11 +30,24 @@ from zope.schema.interfaces import IField
 
 def superAdapter(specific_interface, adapter, objects, name=u""):
     """Find the next most specific adapter.
-    """
 
-    #  We are adjusting view object class to provide IForm rather than IEasyFormForm or IGroup to make
-    #  one of the objects less specific. This allows us to find anotehr adapter other than our one. This allows us to
-    #  find any custom adapters for any fields that we have overridden
+    This is called by a FieldExtenderValidator or FieldExtenderDefault instance.
+    This is passed in with the 'adapter' parameter.
+    This adapter itself is not a real validator or default factory,
+    but is used to find other real validators or default factories.
+
+    This may sound strange, but it solves a problem.
+    Problem is that validators and default were not always found for fields in field sets.
+    For example, a captcha field in the main form would get validated by its proper validator,
+    but when in a field set, only a basic validator would be found.
+
+    We are adjusting the view object class to provide IForm rather than
+    IEasyFormForm or IGroup to make one of the objects less specific.
+    Same for the default factory.
+    This allows us to find another adapter other than the current one.
+    This allows us to find any custom adapters for any fields that we have overridden
+
+    """
     new_obj = []
     found = False
     for obj in objects:
@@ -59,18 +73,24 @@ def superAdapter(specific_interface, adapter, objects, name=u""):
     if not found:
         return None
 
-    provided_by_declared = providedBy(adapter).declared
-    if not provided_by_declared:
-        return None
+    provided_by = providedBy(adapter)
+    # With zope.interface 5.0.2, the info we seek is in 'declared'.
+    # With 5.1.0+, it can also be in the 'interfaces()' iterator,
+    # especially for groups (field sets).
+    # But it looks like interfaces() works always.
+    # adapter_interfaces = provided_by.declared
+    # if not adapter_interfaces:
+    adapter_interfaces = list(provided_by.interfaces())
+    if not adapter_interfaces:
+        return
 
-    return queryMultiAdapter(new_obj, provided_by_declared[0], name=name)
+    return queryMultiAdapter(new_obj, adapter_interfaces[0], name=name)
 
 
 @implementer(IValidator)
 @adapter(IEasyForm, Interface, IEasyFormForm, IField, Interface)
 class FieldExtenderValidator(object):
-    """z3c.form validator class for easyform fields in the default fieldset
-    """
+    """z3c.form validator class for easyform fields in the default fieldset"""
 
     def __init__(self, context, request, view, field, widget):
         self.context = context
@@ -80,8 +100,7 @@ class FieldExtenderValidator(object):
         self.widget = widget
 
     def validate(self, value):
-        """Validate field by TValidator
-        """
+        """Validate field by TValidator"""
         # By default this will call SimpleFieldValidator.validator but allows for a fields
         # custom validation adaptor to also be called such as recaptcha
         _, _, view_interface, _, _ = self.__class__.__component_adapts__
@@ -100,7 +119,7 @@ class FieldExtenderValidator(object):
                 vmethod = queryUtility(IFieldValidator, name=validator)
                 if not vmethod:
                     continue
-                res = vmethod(value)
+                res = vmethod(value, REQUEST=self.request)
                 if res:
                     raise Invalid(res)
         TValidator = getattr(efield, "TValidator", None)
@@ -116,8 +135,7 @@ class FieldExtenderValidator(object):
 @implementer(IValidator)
 @adapter(IEasyForm, Interface, IGroup, IField, Interface)
 class GroupFieldExtenderValidator(FieldExtenderValidator):
-    """z3c.form validator class for easyform fields in fieldset groups
-    """
+    """z3c.form validator class for easyform fields in fieldset groups"""
 
     pass
 
@@ -125,8 +143,7 @@ class GroupFieldExtenderValidator(FieldExtenderValidator):
 @implementer(IValue)
 @adapter(IEasyForm, Interface, IEasyFormForm, IField, Interface)
 class FieldExtenderDefault(object):
-    """z3c.form default class for easyform fields in the default fieldset
-    """
+    """z3c.form default class for easyform fields in the default fieldset"""
 
     def __init__(self, context, request, view, field, widget):
         self.context = context
@@ -136,8 +153,7 @@ class FieldExtenderDefault(object):
         self.widget = widget
 
     def get(self):
-        """Get default value of field from TDefault
-        """
+        """Get default value of field from TDefault"""
         efield = IFieldExtender(self.field)
         TDefault = getattr(efield, "TDefault", None)
         if TDefault:
@@ -161,30 +177,26 @@ class FieldExtenderDefault(object):
 @implementer(IValue)
 @adapter(IEasyForm, Interface, IGroup, IField, Interface)
 class GroupFieldExtenderDefault(FieldExtenderDefault):
-    """z3c.form default class for easyform fields in fieldset groups
-    """
+    """z3c.form default class for easyform fields in fieldset groups"""
 
     pass
 
 
 @implementer(IFromUnicode, ILabel)
 class Label(Field):
-    """A Label field
-    """
+    """A Label field"""
 
     def validate(self, value):
         pass
 
     def fromUnicode(self, str):
-        """
-        """
+        """ """
         return
 
 
 @implementer(IRichLabel)
 class RichLabel(Label):
-    """A Rich Label field
-    """
+    """A Rich Label field"""
 
     rich_label = u""
 
@@ -204,8 +216,7 @@ RichLabelHandler = BaseHandler(RichLabel)
 
 @implementer(IReCaptcha)
 class ReCaptcha(TextLine):
-    """A ReCaptcha field
-    """
+    """A ReCaptcha field"""
 
 
 ReCaptchaFactory = FieldFactory(
@@ -213,11 +224,20 @@ ReCaptchaFactory = FieldFactory(
 )
 ReCaptchaHandler = BaseHandler(ReCaptcha)
 
+@implementer(IHCaptcha)
+class HCaptcha(TextLine):
+    """A HCaptcha field"""
+
+
+HCaptchaFactory = FieldFactory(
+    HCaptcha, _(u"label_hcaptcha_field", default=u"HCaptcha")
+)
+HCaptchaHandler = BaseHandler(HCaptcha)
+
 
 @implementer(INorobotCaptcha)
 class NorobotCaptcha(TextLine):
-    """A NorobotCaptcha field
-    """
+    """A NorobotCaptcha field"""
 
 
 NorobotFactory = FieldFactory(
